@@ -1,9 +1,10 @@
 use crate::initiate_shutdown;
-use tokio;
+
+use std::panic;
 
 /// Waits for a signal that requests a graceful shutdown, like SIGTERM or SIGINT.
 #[cfg(unix)]
-fn wait_for_signal() {
+async fn wait_for_signal() {
     use tokio::signal::unix::{signal, SignalKind};
 
     let mut signal_terminate = signal(SignalKind::terminate()).unwrap();
@@ -28,7 +29,14 @@ async fn wait_for_signal() {
     initiate_shutdown();
 }
 
-/// Registers Ctrl+C and SIGTERM handlers to cause a program shutdown
+/// Registers Ctrl+C and SIGTERM handlers to cause a program shutdown.
+/// Further, registers a custom panic handler to also initiate a shutdown.
+/// Otherwise, a multi-threaded system would deadlock on panik.
 pub fn register_signal_handlers() {
+    panic::set_hook(Box::new(|panic_info| {
+        log::error!("ERROR: {}", panic_info);
+        initiate_shutdown();
+    }));
+
     tokio::spawn(wait_for_signal());
 }
